@@ -4,7 +4,7 @@
 
 ;; Author: Eric M. Ludlam <zappo@gnu.org>
 ;; Keywords: syntax
-;; X-RCS: $Id: semantic-util.el,v 1.3 2000/10/19 02:59:40 youngs Exp $
+;; X-RCS: $Id: semantic-util.el,v 1.32 2000/10/18 02:47:13 zappo Exp $
 
 ;; This file is not part of GNU Emacs.
 
@@ -338,7 +338,7 @@ in the new list."
 		       (set-buffer streamorbuffer)
 		       (semantic-bovinate-toplevel))
 		   streamorbuffer))
-	(nl nil) (ts nil))
+	(nl nil))
     (while stream
       (if (funcall function (car stream))
 	  (setq nl (cons (car stream) nl)))
@@ -355,7 +355,7 @@ in the new list."
 		       (set-buffer streamorbuffer)
 		       (semantic-bovinate-toplevel))
 		   streamorbuffer))
-	(ts nil) (found nil))
+	(found nil))
     (while (and (not found) stream)
       (if (funcall function (car stream))
 	  (setq found (car stream)))
@@ -667,7 +667,7 @@ depended on, and functions will move to the specified definition."
 	     (if f (find-file f))))
       nil
     (let ((s (semantic-fetch-overload 'find-nonterminal)))
-      (if s (funcall s token)
+      (if s (funcall s token parent)
 	(set-buffer (semantic-token-buffer token))
 	(let ((start (semantic-token-start token)))
 	  (if (numberp start)
@@ -694,7 +694,7 @@ If nosnarf if 'flex, then only return the flex token."
   (if (not token)
       (setq token (car (semantic-find-nonterminal-by-overlay nil))))
   (let ((s (semantic-fetch-overload 'find-documentation)))
-    (if s (funcall s token)
+    (if s (funcall s token nosnarf)
       ;; No override.  Try something simple to find documentation nearby
       (save-excursion
 	(set-buffer (semantic-token-buffer token))
@@ -908,7 +908,8 @@ instead of read-only."
 
 ;;; Interactive Functions for bovination
 ;;
-(eval-when-compile (require 'pp))
+(eval-when-compile
+  (condition-case nil (require 'pp) (error nil)))
 
 (defun bovinate (&optional clear)
   "Bovinate the current buffer.  Show output in a temp buffer.
