@@ -5,7 +5,7 @@
 ;; Author: Eric M. Ludlam <zappo@gnu.org>
 ;; Version: 1.3
 ;; Keywords: syntax
-;; X-RCS: $Id: semantic.el,v 1.3 2000/10/19 02:59:40 youngs Exp $
+;; X-RCS: $Id: semantic.el,v 1.59 2000/10/18 03:27:05 zappo Exp $
 
 ;; This file is not part of GNU Emacs.
 
@@ -215,7 +215,6 @@
 
 (require 'working)
 (require 'semantic-util)
-(require 'overlay)
 
 (defgroup semantic nil
   "File and tag browser frame."
@@ -225,31 +224,36 @@
 
 ;;; Compatibility
 ;;
-(if (fboundp 'make-overlay)
+(if (featurep 'xemacs)
     (progn
-      (defalias 'semantic-make-overlay 'make-overlay)
-      (defalias 'semantic-overlay-put 'overlay-put)
-      (defalias 'semantic-overlay-get 'overlay-get)
-      (defalias 'semantic-overlay-delete 'delete-overlay)
-      (defalias 'semantic-overlays-at 'overlays-at)
-      (defalias 'semantic-overlays-in 'overlays-in)
-      (defalias 'semantic-overlay-buffer 'overlay-buffer)
-      (defalias 'semantic-overlay-start 'overlay-start)
-      (defalias 'semantic-overlay-end 'overlay-end)
-      (defalias 'semantic-overlay-next-change 'next-overlay-change)
-      (defalias 'semantic-overlay-previous-change 'previous-overlay-change)
+      (defalias 'semantic-overlay-live-p 'extent-live-p)
+      (defalias 'semantic-make-overlay 'make-extent)
+      (defalias 'semantic-overlay-put 'set-extent-property)
+      (defalias 'semantic-overlay-get 'extent-property)
+      (defalias 'semantic-overlay-delete 'delete-extent)
+      (defalias 'semantic-overlays-at 'extents-at)
+      (defalias 'semantic-overlays-in 'extents-in)
+      (defalias 'semantic-overlay-buffer 'extent-buffer)
+      (defalias 'semantic-overlay-start 'extent-start-position)
+      (defalias 'semantic-overlay-end 'extent-end-position)
+      (defalias 'semantic-overlay-next-change 'next-extent-change)
+      (defalias 'semantic-overlay-previous-change 'previous-extent-change)
+      (defalias 'semantic-overlay-lists
+	(lambda () (list (extent-list))))
       )
-  (defalias 'semantic-make-overlay 'make-extent)
-  (defalias 'semantic-overlay-put 'set-extent-property)
-  (defalias 'semantic-overlay-get 'get-extent-property)
-  (defalias 'semantic-overlay-delete 'delete-extent)
-  (defalias 'semantic-overlays-at 'extents-at)
-  (defalias 'semantic-overlays-in 'extents-in)
-  (defalias 'semantic-overlay-buffer 'extent-buffer)
-  (defalias 'semantic-overlay-start 'extent-start)
-  (defalias 'semantic-overlay-end 'extent-end)
-  (defalias 'semantic-overlay-next-change 'next-extent-change)
-  (defalias 'semantic-overlay-previous-change 'previous-extent-change)
+  (defalias 'semantic-overlay-live-p 'overlay-buffer)
+  (defalias 'semantic-make-overlay 'make-overlay)
+  (defalias 'semantic-overlay-put 'overlay-put)
+  (defalias 'semantic-overlay-get 'overlay-get)
+  (defalias 'semantic-overlay-delete 'delete-overlay)
+  (defalias 'semantic-overlays-at 'overlays-at)
+  (defalias 'semantic-overlays-in 'overlays-in)
+  (defalias 'semantic-overlay-buffer 'overlay-buffer)
+  (defalias 'semantic-overlay-start 'overlay-start)
+  (defalias 'semantic-overlay-end 'overlay-end)
+  (defalias 'semantic-overlay-next-change 'next-overlay-change)
+  (defalias 'semantic-overlay-previous-change 'previous-overlay-change)
+  (defalias 'semantic-overlay-lists 'overlay-lists)
   )
 
 (defvar semantic-edebug nil
@@ -340,7 +344,7 @@ a type nonterminal.
 
 TOP-LEVEL ENTRIES:
 
- (\"NAME\" variable \"TYPE\" CONST DEFAULT-VALUE MODIFIERS [OPTSUFFIX] 
+ (\"NAME\" variable \"TYPE\" CONST DEFAULT-VALUE MODIFIERS [OPTSUFFIX]
            \"DOCSTRING\" OVERLAY)
    The definition of a variable, or constant.
    CONST is a boolean representing if this variable is considered a constant.
@@ -402,11 +406,13 @@ string can be replaced with `Imports'.")
 (defvar semantic-flex-depth 0
   "Default flexing depth.
 This specifies how many lists to create tokens in.")
+(make-variable-buffer-local 'semantic-flex-depth)
 
 (defvar semantic-ignore-comments t
   "Default comment handling.
 t means to strip comments when flexing.  Nil means to keep comments
 as part of the token stream.")
+(make-variable-buffer-local 'semantic-ignore-comments)
 
 (defvar semantic-expand-nonterminal nil
   "Function to call for each returned Non-terminal.
@@ -430,6 +436,25 @@ This is tracked with `semantic-change-function'.")
 This function should behave as the function `semantic-bovinate-toplevel'.")
 (make-variable-buffer-local 'semantic-toplevel-bovinate-override)
 
+(defvar semantic-after-toplevel-bovinate-hook nil
+  "Hooks run after a toplevel token parse.
+It is not run if the toplevel parse command is called, and buffer does
+not need to be reparsed.
+For language specific hooks, make sure you define this as a local hook.")
+
+(defvar semantic-before-toplevel-cache-flush-hook nil
+  "Hooks run before the toplevel nonterminal cache is flushed.
+For language specific hooks, make sure you define this as a local hook.")
+
+(defvar semantic-reparse-needed-change-hook nil
+  "Hooks run when a user edit is detected as needing a reparse.
+For language specific hooks, make sure you define this as a local hook.")
+
+(defvar semantic-no-reparse-needed-change-hook nil
+  "Hooks run when a user edit is detected as not needing a reparse.
+If the hook returns non-nil, then declare that a reparse is needed.
+For language specific hooks, make sure you define this as a local hook.")
+
 
 ;;; Utility API functions
 ;;
@@ -513,10 +538,11 @@ If not provided, then only the POSITION can be provided."
 (defun semantic-clear-toplevel-cache ()
   "Clear the toplevel bovin cache for the current buffer."
   (interactive)
+  (run-hooks 'semantic-before-toplevel-cache-flush-hook)
   (setq semantic-toplevel-bovine-cache nil)
   ;; Nuke all semantic overlays.  This is faster than deleting based
   ;; on our data structure.
-  (let ((l (overlay-lists)))
+  (let ((l (semantic-overlay-lists)))
     (mapcar 'semantic-delete-overlay-maybe (car l))
     (mapcar 'semantic-delete-overlay-maybe (cdr l))
     )
@@ -534,32 +560,35 @@ there has been a size change."
 	   checkcache
 	   semantic-toplevel-bovine-cache-check)
       (semantic-clear-toplevel-cache))
-  (cond
-   (semantic-toplevel-bovinate-override
-    (funcall semantic-toplevel-bovinate-override checkcache))
-   ((and semantic-toplevel-bovine-cache
-	 (car semantic-toplevel-bovine-cache)
-	 ;; Add a rule that knows how to see if there have
-	 ;; been "big chagnes"
-	 )
-    (car semantic-toplevel-bovine-cache))
-   (t
-    (let ((ss (semantic-flex (point-min) (point-max)))
-	  (res nil)
-	  (semantic-overlay-error-recovery-stack nil))
-      ;; Init a dump
-      (if semantic-dump-parse (semantic-dump-buffer-init))
-      ;; Parse!
-      (working-status-forms (buffer-name) "done"
-	(setq res
-	      (semantic-bovinate-nonterminals
-	       ss 'bovine-toplevel semantic-flex-depth))
-	(working-status t))
-      (setq semantic-toplevel-bovine-cache
-	    (list (nreverse res) (point-max))
-	    semantic-toplevel-bovine-cache-check nil)
-      (add-hook 'after-change-functions 'semantic-change-function nil t)
-      (car semantic-toplevel-bovine-cache)))))
+  (prog1
+      (cond
+       (semantic-toplevel-bovinate-override
+	(funcall semantic-toplevel-bovinate-override checkcache))
+       ((and semantic-toplevel-bovine-cache
+	     (car semantic-toplevel-bovine-cache)
+	     ;; Add a rule that knows how to see if there have
+	     ;; been "big chagnes"
+	     )
+	(car semantic-toplevel-bovine-cache))
+       (t
+	(let ((ss (semantic-flex (point-min) (point-max)))
+	      (res nil)
+	      (semantic-overlay-error-recovery-stack nil))
+	  ;; Init a dump
+	  (if semantic-dump-parse (semantic-dump-buffer-init))
+	  ;; Parse!
+	  (working-status-forms (buffer-name) "done"
+	    (setq res
+		  (semantic-bovinate-nonterminals
+		   ss 'bovine-toplevel semantic-flex-depth))
+	    (working-status t))
+	  (setq semantic-toplevel-bovine-cache
+		(list (nreverse res) (point-max))
+		semantic-toplevel-bovine-cache-check nil)
+	  (add-hook 'after-change-functions 'semantic-change-function nil t)
+	  (run-hooks 'semantic-after-toplevel-bovinate-hook)
+	  (car semantic-toplevel-bovine-cache))))
+    ))
 
 (defun semantic-change-function (start end length)
   "Run whenever a buffer controlled by `semantic-mode' change.
@@ -580,15 +609,23 @@ Argument START, END, and LENGTH specify the bounds of the change."
 	      (if (and (eq (semantic-token-token (car tl)) 'type)
 		       (not (cdr tl))
 		       (semantic-token-type-parts (car tl)))
-		  ;; This is between two items in a type with
-		  ;; stuff in it.
-		  (setq semantic-toplevel-bovine-cache-check t)
-		;; This is ok, chuck it.
-		nil))
+		  (progn
+		    ;; This is between two items in a type with
+		    ;; stuff in it.
+		    (setq semantic-toplevel-bovine-cache-check t)
+		    (run-hooks 'semantic-reparse-needed-change-hook))
+		;; This is might be ok, chuck it.
+		(if (run-hooks 'semantic-no-reparse-needed-change-hook)
+		    (progn
+		      ;; The hook says so, so flush it.
+		      (setq semantic-toplevel-bovine-cache-check t)
+		      (run-hooks 'semantic-reparse-needed-change-hook))
+		  nil)))
 	     ;; If we  cover the beginning or end of this item, we must
 	     ;; reparse this object.
 	     (t
-	      (setq semantic-toplevel-bovine-cache-check t)))
+	      (setq semantic-toplevel-bovine-cache-check t)
+	      (run-hooks 'semantic-reparse-needed-change-hook)))
 	    ;; next
 	    (setq tl (cdr tl)))
 	;; There was no hit, perhaps we need to reparse this intermediate area.
@@ -646,13 +683,15 @@ the current results on a parse error."
 					     (car motok))
 			    ;; this will support new overlays created by
 			    ;; the special function, or recycles
-			    start (or (semantic-overlay-start
-				       (car startcdr)) start)
-			    end (or (semantic-overlay-end
-				     (car startcdr)) end)
+			    start (if (semantic-overlay-live-p (car startcdr))
+				      (semantic-overlay-start (car startcdr))
+				    start)
+			    end (if (semantic-overlay-live-p (car startcdr))
+				    (semantic-overlay-end (car startcdr))
+				  end)
 			    o (semantic-make-overlay start end
 						     (current-buffer)))
-		      (if (semantic-overlay-buffer (car startcdr))
+		      (if (semantic-overlay-live-p (car startcdr))
 			  (semantic-overlay-delete (semantic-token-overlay
 						    (car motok))))
 		      (semantic-overlay-stack-add o)
@@ -1086,7 +1125,7 @@ These keywords are matched explicitly, and converted into special symbols.")
   "Updates to the syntax table for this buffer.
 These changes are active only while this file is being flexed.
 This is a list where each element is of the form:
-  (CHAR CLASS) 
+  (CHAR CLASS)
 Where CHAR is the char passed to `modify-syntax-entry',
 and CLASS is the string also passed to `modify-syntax-entry' to define
 what class of syntax CHAR is.")
